/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.cdaimport;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.uhn.fhir.interceptor.api.Interceptor;
import com.smilecdr.demo.channelimport.ChannelImportInterceptorTemplate;
import org.hl7.fhir.instance.model.api.IBaseBundle;
import org.hl7.fhir.instance.model.api.IBaseOperationOutcome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Sample Channel Import Interceptor implementing all CDA Import / Export pointcuts.
 * It is intended to be used in CDA EXCHANGE module 'Interceptor Bean Types'.
 * Can be used as a starting point for your channel import interceptor.
 */
@SuppressWarnings({"unused", "EmptyTryBlock"})
@Interceptor
public class CDAImportInterceptorTemplate {

	private static final Logger ourLog = LoggerFactory.getLogger(ChannelImportInterceptorTemplate.class);

	public CDAImportInterceptorTemplate() {
	}

	@CdrHook(CdrPointcut.CDA_PRE_IMPORT)
	public String cdaImportPreImport(String theDocumentToImport) {
		ourLog.info("Interceptor CDA_PRE_IMPORT - started");
		try {
			// Document processing here...
		} finally {
			ourLog.info("Interceptor CDA_PRE_IMPORT - ended");
		}
		return theDocumentToImport;
	}

	@CdrHook(CdrPointcut.CDA_POST_IMPORT)
	public void cdaImportPostImport(IBaseOperationOutcome theOutcome, IBaseBundle theBundle, String theImportedDocument) {
		ourLog.info("Interceptor CDA_POST_IMPORT - started");
		try {
			// Document and bundle processing here...
		} finally {
			ourLog.info("Interceptor CDA_POST_IMPORT - ended");
		}
	}

	@CdrHook(CdrPointcut.CDA_POST_EXPORT)
	public String cdaExportPostExport(IBaseBundle theBundle, String theExportedDocument) {
		ourLog.info("Interceptor CDA_PRE_EXPORT - started");
		try {
			// Document processing here...
		} finally {
			ourLog.info("Interceptor CDA_PRE_EXPORT - ended");
		}
		return theExportedDocument;
	}
}
