/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.fhirgateway;

import ca.cdr.api.broker.IPublicBrokerSender;
import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.fhirgw.json.AvailableRoutesJson;
import ca.cdr.api.fhirgw.json.GatewayTargetJson;
import ca.cdr.api.fhirgw.json.MatchedRoutesJson;
import ca.cdr.api.fhirgw.model.ISearchResultsAccumulator;
import ca.cdr.api.fhirgw.model.OperationRequest;
import ca.cdr.api.fhirgw.model.OperationResponse;
import ca.cdr.api.fhirgw.model.ReadRequest;
import ca.cdr.api.fhirgw.model.SearchPageRequest;
import ca.cdr.api.fhirgw.model.SearchRequest;
import ca.cdr.api.fhirgw.model.SearchResponse;
import ca.cdr.api.fhirgw.model.UpdateRequest;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.rest.server.servlet.ServletRequestDetails;
import ca.uhn.fhir.util.StopWatch;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Sample Gateway Interceptor implementing all Smile CDR FHIRGW_XXX pointcuts.
 * It is intended to be used in FHIR Gateway REST Endpoint module 'Interceptor Bean Types'.
 * Can be used as a starting point for your Gateway interceptor.
 */
@SuppressWarnings({"unused", "EmptyTryBlock"})
@Interceptor
public class GatewayInterceptorTemplate {


	@Autowired
	private IPublicBrokerSender myBrokerSender;

	private static final Logger ourLog = LoggerFactory.getLogger(GatewayInterceptorTemplate.class);

	@CdrHook(CdrPointcut.FHIRGW_READ_TARGET_PREINVOKE)
	public void fhirgwReadTargetPreInvoke(
		ReadRequest theReadRequest,
		GatewayTargetJson theGatewayTargetJson,
		ServletRequestDetails theServletRequestDetails) {

		ourLog.info("Interceptor FHIRGW_READ_TARGET_PREINVOKE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_READ_TARGET_PREINVOKE - ended, execution took {}", stopWatch);
		}
	}

	@CdrHook(CdrPointcut.FHIRGW_SEARCH_TARGET_PREINVOKE)
	public void fhirgwSearchTargetPreInvoke(
		SearchRequest theSearchRequest,
		GatewayTargetJson theGatewayTargetJson,
		ISearchResultsAccumulator theISearchResultsAccumulator,
		ServletRequestDetails theServletRequestDetails) {

		ourLog.info("Interceptor FHIRGW_SEARCH_TARGET_PREINVOKE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_SEARCH_TARGET_PREINVOKE - ended, execution took {}", stopWatch);
		}
	}

	@CdrHook(CdrPointcut.FHIRGW_SEARCH_PAGE_TARGET_PREINVOKE)
	public void fhirgwSearchPageTargetPreInvoke(
		SearchPageRequest theSearchPageRequest,
		GatewayTargetJson theGatewayTargetJson,
		ISearchResultsAccumulator theISearchResultsAccumulator,
		ServletRequestDetails theServletRequestDetails) {

		ourLog.info("Interceptor FHIRGW_SEARCH_PAGE_TARGET_PREINVOKE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_SEARCH_PAGE_TARGET_PREINVOKE - ended, execution took {}", stopWatch);
		}
	}

	@CdrHook(CdrPointcut.FHIRGW_SEARCH_TARGET_POSTINVOKE)
	public void fhirgwSearchTargetPostInvoke(
		GatewayTargetJson theGatewayTargetJson,
		ISearchResultsAccumulator theISearchResultsAccumulator,
		SearchResponse theSearchResponse,
		ServletRequestDetails theServletRequestDetails) {

		ourLog.info("Interceptor FHIRGW_SEARCH_TARGET_POSTINVOKE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_SEARCH_TARGET_POSTINVOKE - ended, execution took {}", stopWatch);
		}
	}

	@CdrHook(CdrPointcut.FHIRGW_OPERATION_TARGET_PREINVOKE)
	public void fhirgwOperationTargetPreInvoke(
		OperationRequest theOperationRequest,
		GatewayTargetJson theGatewayTargetJson,
		ServletRequestDetails theServletRequestDetails) {

		ourLog.info("Interceptor FHIRGW_OPERATION_TARGET_PREINVOKE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_OPERATION_TARGET_PREINVOKE - ended, execution took {}", stopWatch);
		}
	}

	@CdrHook(CdrPointcut.FHIRGW_OPERATION_TARGET_POSTINVOKE)
	public void fhirgwOperationTargetPostinvoke(
		OperationRequest theOperationRequest,
		ISearchResultsAccumulator theISearchResultsAccumulator,
		OperationResponse theOperationResponse,
		GatewayTargetJson theGatewayTargetJson,
		ServletRequestDetails theServletRequestDetails) {

		ourLog.info("Interceptor FHIRGW_OPERATION_TARGET_POSTINVOKE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_OPERATION_TARGET_POSTINVOKE - ended, execution took {}", stopWatch);
		}
	}

	@CdrHook(CdrPointcut.FHIRGW_UPDATE_POST_SELECT_ROUTE)
	public void fhirgwUpdatePostSelectRoute(
		UpdateRequest theUpdateRequest,
		ServletRequestDetails theRequestDetails,
		MatchedRoutesJson theMatchedRoutes,
		AvailableRoutesJson theAvailableRoutes) {
		ourLog.info("interceptor FHIRGW_UPDATE_POST_SELECT_ROUTE - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor FHIRGW_UPDATE_POST_SELECT_ROUTE - ended, execution took {}", stopWatch);
		}
	}

}
