/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.hl7v2listening;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.model.json.ConvertedTransactionBundlesJson;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.util.StopWatch;
import ca.uhn.hl7v2.model.Message;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Sample HL7V2 Listening Endpoint Interceptor implementing all currently available pointcuts.
 * It is intended to be used in HL7V2 Listening Endpoint module's 'Interceptor Bean Types'.
 * Can be used as a starting point for your HL7V2 Listening Endpoint Interceptor. Note that any work done here can easily
 * be done in the javascript framework as well.
 */
@SuppressWarnings({"unused", "EmptyTryBlock"})
@Interceptor
public class Hl7V2ListeningInterceptorTemplate {

	private static final Logger ourLog = LoggerFactory.getLogger(Hl7V2ListeningInterceptorTemplate.class);

	@CdrHook(CdrPointcut.HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING)
	public Boolean channelImportMessagePreProcessed(Message theMessage) {

		ourLog.info("Interceptor HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		}
		finally {
			ourLog.info("Interceptor HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING - ended, execution took {}", stopWatch);
		}
		// should return a boolean indicating whether to proceed
		return true;
	}

	@CdrHook(CdrPointcut.HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING)
	public void channelImportMessagePreProcessed(Message theMessage, ConvertedTransactionBundlesJson theConvertedBundles) {

		ourLog.info("Interceptor HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING - started");
		StopWatch stopWatch = new StopWatch();
		try {
			//Cast to your STU-Specific bundle and do work here, e.g. if you are using R4:
			//IBaseBundle iBaseBundle = theConvertedBundles.getTransactionBundles().get(0);
			//org.hl7.fhir.r4.model.Bundle r4bundle = (org.hl7.fhir.r4.model.Bundle) iBaseBundle;
			//bundle.getEntry().stream().forEach(bundleEntry -> ourLog.info("Bundle Entry: {}", bundleEntry.getResource().getResourceType())

		}
		finally {
			ourLog.info("Interceptor HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING - ended, execution took {}", stopWatch);
		}
	}
}
