/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.mdm;

import ca.uhn.fhir.interceptor.api.Hook;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.interceptor.api.Pointcut;
import ca.uhn.fhir.mdm.model.mdmevents.MdmLinkEvent;
import ca.uhn.fhir.rest.server.TransactionLogMessages;
import ca.uhn.fhir.rest.server.messaging.ResourceOperationMessage;
import ca.uhn.fhir.util.StopWatch;
import org.hl7.fhir.instance.model.api.IBaseResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Sample MDM Interceptor implementing all MDM_XXX pointcuts.
 * It is intended to be used in MDM module 'Interceptor Bean Types'.
 * Can be used as a starting point for your mdm interceptor.
 */
@SuppressWarnings({"unused", "EmptyTryBlock"})
@Interceptor
public class MDMInterceptorTemplate {

	private static final Logger ourLog = LoggerFactory.getLogger(MDMInterceptorTemplate.class);

	@Hook(Pointcut.MDM_BEFORE_PERSISTED_RESOURCE_CHECKED)
	public void mdmBeforePersistedResourceChecked(
		IBaseResource theResource) {

		ourLog.info("Interceptor MDM_BEFORE_PERSISTED_RESOURCE_CHECKED - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor MDM_BEFORE_PERSISTED_RESOURCE_CHECKED - ended, execution took {}", stopWatch);
		}
	}

	@Hook(value = Pointcut.MDM_AFTER_PERSISTED_RESOURCE_CHECKED)
	public void mdmAfterPersistedResourceChecked(
		ResourceOperationMessage theResourceOperationMessage,
		TransactionLogMessages theTransactionLogMessages,
		MdmLinkEvent theMdmLinkEvent) {

		ourLog.info("Interceptor MDM_AFTER_PERSISTED_RESOURCE_CHECKED - started");
		StopWatch stopWatch = new StopWatch();
		try {
			// your implementation goes here
		} finally {
			ourLog.info("Interceptor MDM_AFTER_PERSISTED_RESOURCE_CHECKED - ended, execution took {}", stopWatch);
		}
	}
}
