// Global handler
// This function will be called by MDM module to determine which attributes are replaced
// and which are preserved in the golden record.
// It modifies the current golden record based on the target record found.
function mdmApplySurvivorshipRules(targetRec, goldenRec, txContext) {
    // Replaces all values in the golden resource with the values in the target resource.
    // But IDs, identifiers and metadata are not replaced.
    var helper = new MdmHelper(Fhir.getContext(), targetRec, goldenRec, txContext);
    helper.replaceAll();
    // Now copy new identifiers and update current ones per system in the golden record.
    for (let i = 0; i < targetRec.getIdentifier().size(); i++) {
        let targetId = targetRec.getIdentifier().get(i);
        let goldenId = getIdentifier(goldenRec, targetId.getSystem());
        if (goldenId === null) {
            goldenRec.addIdentifier(targetId.copy());
        } else {
            goldenId.setValue(targetId.getValue());
        }
    }
}

function getIdentifier(resource, system) {
    // get first identifier using the system from resource identifiers
    for (let i = 0; i < resource.getIdentifier().size(); i++) {
        var id = resource.getIdentifier().get(i);
        if (id.getSystem().equals(system)) {
            return id;
        }
    }
    return null;
}

// This function will be called by MDM module to merge two golden records, when needed.
function mdmApplySurvivorshipRulesOnMergeGoldenResources(targetRec, goldenRec, transactionContext) {
   // merge all fields by default
   var helper = new MdmHelper(Fhir.getContext(), targetRec, goldenRec, transactionContext);
   helper.mergeAll();
}
